#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (C) 2011-2014 Star2Billing S.L.
#
# The Initial Developer of the Original Code is
# Arezqui Belaid <info@star2billing.com>
#
import datetime
from collections import OrderedDict
from unittest import skipIf

import django
from django.conf import settings
from django.contrib.auth.models import Permission
from django.core.exceptions import ValidationError
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse
from django.utils import timezone
from model_mommy import mommy

from admin_tools_stats.models import DashboardStatsCriteria

from .utils import (BaseSuperuserAuthenticatedClient,
                    BaseUserAuthenticatedClient, assertContainsAny)


class AdminToolsStatsAdminInterfaceTestCase(BaseSuperuserAuthenticatedClient):
    """
    Test cases for django-admin-tools-stats Admin Interface
    """

    def test_admin_tools_stats_dashboardstats(self):
        """Test function to check dashboardstats admin pages"""
        response = self.client.get('/admin/admin_tools_stats/')
        self.assertEqual(response.status_code, 200)
        response = self.client.get('/admin/admin_tools_stats/dashboardstats/')
        self.assertEqual(response.status_code, 200)

    def test_admin_tools_stats_dashboardstatscriteria(self):
        """Test function to check dashboardstatscriteria admin pages"""
        response = \
            self.client.get('/admin/admin_tools_stats/dashboardstatscriteria/')
        self.assertEqual(response.status_code, 200)


class AdminToolsStatsAdminCharts(BaseSuperuserAuthenticatedClient):
    def test_admin_dashboard_page(self):
        """Test function to check dashboardstatscriteria admin pages"""
        stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            graph_title="User graph",
            model_name="User",
            model_app_name="auth",
        )
        mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            graph_title="User logged in graph",
            model_name="User",
            model_app_name="auth",
        )
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "": [None, "All"],
                "false": [False, "Inactive"],
                "true": [True, "Active"],
            },
        )
        mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats)
        response = self.client.get('/admin/')
        self.assertContains(
            response,
            '<h2>User graph</h2>',
            html=True,
        )
        self.assertContains(
            response,
            '<h2>User logged in graph</h2>',
            html=True,
        )
        self.assertContains(
            response,
            '<svg style="width:600px;height:400px;"></svg>',
            html=True,
        )
        self.assertContains(
            response,
            '<option value="True">Active</option>',
            html=True,
        )
        self.assertContains(
            response,
            '<option value="False">Inactive</option>',
            html=True,
        )

    def test_admin_dashboard_page_multi_series(self):
        stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
        )
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "": [None, "All"],
                "false": [False, "Inactive"],
                "true": [True, "Active"],
            },
        )
        cm2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats, use_as='multiple_series')
        stats.default_multiseries_criteria = cm2m
        stats.save()
        response = self.client.get('/admin/')
        self.assertContains(
            response,
            '<select name="select_box_multiple_series" class="chart-input select_box_multiple_series" required>'
            '<option value="">-------</option>'
            '<option value="2" selected>active</option>'
            '</select>',
            html=True,
        )

    def test_admin_dashboard_page_post(self):
        """Test function to check dashboardstatscriteria admin pages"""
        stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
        )
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "": [None, "All"],
                "false": [False, "Inactive"],
                "true": [True, "Active"],
            },
        )
        mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats)
        response = self.client.post('/admin/', {'select_box_user_graph': 'true'})
        self.assertContains(
            response,
            '<input type="hidden" class="hidden_graph_key" name="graph_key" value="user_graph">',
            html=True,
        )
        self.assertContains(
            response,
            '<option value="True">Active</option>',
            html=True,
        )


class ModelTests(TestCase):
    maxDiff = None

    def setUp(self):
        self.stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
        )
        self.kid_stats = mommy.make(
            'DashboardStats',
            date_field_name="birthday",
            model_name="TestKid",
            model_app_name="demoproject",
            graph_key="kid_graph",
        )

    def test_clean_error_model_app_app_name(self):
        stats = mommy.make('DashboardStats', model_name="User1", model_app_name="auth1", graph_key="error_graph")
        with self.assertRaisesRegex(ValidationError, "model_name.*No installed app with label"):
            stats.clean()

    def test_clean_error_model_name(self):
        stats = mommy.make('DashboardStats', model_name="User1", model_app_name="auth", graph_key="error_graph")
        with self.assertRaisesRegex(ValidationError, "model_name.*App 'auth' doesn't have a 'User1' model."):
            stats.clean()

    def test_clean_error_operation_field(self):
        stats = mommy.make('DashboardStats', model_name="User", model_app_name="auth", graph_key="error_graph", operation_field_name='asdf')
        with self.assertRaisesRegex(ValidationError, "operation_field_name.*Cannot resolve keyword 'asdf' into field. Choices are:"):
            stats.clean()

    def test_clean_error_date_field(self):
        stats = mommy.make('DashboardStats', model_name="User", model_app_name="auth", graph_key="error_graph", date_field_name='asdf')
        with self.assertRaisesRegex(ValidationError, "date_field_name.*Cannot resolve keyword 'asdf' into field. Choices are:"):
            stats.clean()

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 10))
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2010, 10, 12)

        interval = "days"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 8, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 9, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 10, 0, 0): {'': 1},
            datetime.datetime(2010, 10, 11, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 12, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_hours(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.datetime(2010, 10, 8, 23, 13))
        time_since = datetime.datetime(2010, 10, 8, 22)
        time_until = datetime.datetime(2010, 10, 8, 23)

        interval = "hours"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 8, 22, 0): {'': 0},
            datetime.datetime(2010, 10, 8, 23, 0): {'': 1},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_weeks(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 30))
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2010, 11, 8)

        interval = "weeks"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 4, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 11, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 18, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 25, 0, 0): {'': 1},
            datetime.datetime(2010, 11, 1, 0, 0): {'': 0},
            datetime.datetime(2010, 11, 8, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_months(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 30))
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2010, 11, 30)

        interval = "months"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 1, 0, 0): {'': 1},
            datetime.datetime(2010, 11, 1, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_quarters(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 30))
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2011, 10, 8)

        interval = "quarters"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 1, 0, 0): {'': 1},
            datetime.datetime(2011, 1, 1, 0, 0): {'': 0},
            datetime.datetime(2011, 4, 1, 0, 0): {'': 0},
            datetime.datetime(2011, 7, 1, 0, 0): {'': 0},
            datetime.datetime(2011, 10, 1, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_years(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 30))
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2011, 10, 8)

        interval = "years"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 1, 1, 0, 0): {'': 1},
            datetime.datetime(2011, 1, 1, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @override_settings(USE_TZ=True, TIME_ZONE='Europe/Prague')
    def test_get_multi_series_datetime_tz(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        current_tz = timezone.get_current_timezone()
        user = mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=current_tz))
        mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, 12, 34, tzinfo=current_tz))
        time_since = datetime.datetime(2010, 10, 9, 0, 0)
        time_until = datetime.datetime(2010, 10, 11, 0, 0)

        interval = "days"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 9, 0, 0).astimezone(current_tz): {'': 0},
            datetime.datetime(2010, 10, 10, 0, 0).astimezone(current_tz): {'': 2},
            datetime.datetime(2010, 10, 11, 0, 0).astimezone(current_tz): {'': 0},
        }
        from pprint import pprint
        pprint(serie)
        pprint("-------------------------------")
        pprint(testing_data)
        self.assertDictEqual(serie, testing_data)

    @override_settings(USE_TZ=True, TIME_ZONE='CET')
    def test_get_multi_series_date_tz(self):
        """Test function to check DashboardStats.get_multi_time_series()"""
        mommy.make('TestKid', birthday=datetime.date(2010, 10, 10))
        mommy.make('TestKid', birthday=None)
        time_since = datetime.datetime(2010, 10, 9)
        time_until = datetime.datetime(2010, 10, 11)

        interval = "days"
        user = mommy.make('User')
        serie = self.kid_stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.date(2010, 10, 9): {'': 0},
            datetime.date(2010, 10, 10): {'': 1},
            datetime.date(2010, 10, 11): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @override_settings(USE_TZ=True, TIME_ZONE='Europe/Prague')
    def test_get_multi_series_change_dst(self):
        """Test function to check DashboardStats.get_multi_time_series() on edge of daylight saving time change """
        current_tz = timezone.get_current_timezone()
        user = mommy.make('User', date_joined=datetime.datetime(2019, 10, 28, tzinfo=current_tz))
        time_since = datetime.datetime(2019, 10, 27, 0, 0)
        time_until = datetime.datetime(2019, 10, 29, 0, 0)

        interval = "days"
        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2019, 10, 27, 0, 0).astimezone(current_tz): {'': 0},
            datetime.datetime(2019, 10, 28, 0, 0).astimezone(current_tz): {'': 1},
            datetime.datetime(2019, 10, 29, 0, 0).astimezone(current_tz): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_distinct_count(self):
        """Test function to check DashboardStats.get_multi_time_series() with distinct count."""
        stats = mommy.make(
            'DashboardStats',
            model_name="User",
            date_field_name='date_joined',
            model_app_name="auth",
            type_operation_field_name="Count",
            distinct=True,
            operation_field_name='first_name',
        )
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Foo")
        mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Foo")
        mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Bar")
        time_since = datetime.datetime(2010, 10, 8)
        time_until = datetime.datetime(2010, 10, 12)

        interval = "days"
        serie = stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 8, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 9, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 10, 0, 0): {'': 2},
            datetime.datetime(2010, 10, 11, 0, 0): {'': 0},
            datetime.datetime(2010, 10, 12, 0, 0): {'': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_distinct_functions(self):
        """Test function to check DashboardStats.get_multi_time_series() with various functions."""
        for func, result in (
                ("Sum", 15),
                ("Avg", 5),
                ("Min", 1),
                ("Max", 12),
                ("StdDev", 4.96655480858378),
                ("Variance", 24.666666666666668),
                ("AvgCountPerInstance", 1),
        ):
            stats = mommy.make(
                'DashboardStats',
                model_name="TestKid",
                date_field_name='birthday',
                model_app_name="demoproject",
                type_operation_field_name=func,
                operation_field_name='age',
            )
            mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=12)
            mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=1)
            mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=2)
            mommy.make('TestKid', birthday=None)
            time_since = datetime.datetime(2010, 10, 9)
            time_until = datetime.datetime(2010, 10, 10)

            interval = "days"
            user = mommy.make('User')
            serie = stats.get_multi_time_series({}, time_since, time_until, interval, None, None, user)
            self.assertEqual(serie[datetime.date(2010, 10, 10)][''], result, "Bad value for function %s" % func)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_dynamic_field_name(self):
        """Test function to check DashboardStats.get_multi_time_series() with dynamic criteria mapping"""
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "": [None, "All"],
                "false": [False, "Inactive"],
                "true": [True, "Active"],
            },
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('Active', 1), ('Inactive', 0))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('Active', 0), ('Inactive', 1))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_dynamic_field_name_old_format(self):
        """
        Test function to check DashboardStats.get_multi_time_series() with dynamic criteria mapping
        the criteria is given in old format
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "False": "Inactive",
                "True": "Active",
            },
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('Active', 1), ('Inactive', 0))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('Active', 0), ('Inactive', 1))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_criteria_without_dynamic_mapping(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        DashboardStatsCriteria is set, but without dynamic mapping, so the values are autogenerated.
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('True', 0), ('False', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('True', 0), ('False', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('True', 1), ('False', 0))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('True', 0), ('False', 1))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('True', 0), ('False', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_criteria_without_dynamic_mapping_choices(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        DashboardStatsCriteria is set, but without dynamic mapping, so the values are autogenerated on CharField.
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="last_name",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo")
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar")
        mommy.make('User', date_joined=datetime.date(2010, 10, 14))
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        user = mommy.make('User', is_superuser=True)
        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('Bar', 0), ('Foo', 1))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('Bar', 1), ('Foo', 0))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_criteria_combine(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        Try to combine multiple_series filter with chart_filter.
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="last_name",
        )
        criteria_active = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        m2m_active = mommy.make('CriteriaToStatsM2M', criteria=criteria_active, stats=self.stats, use_as='chart_filter')
        mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo", is_active=True)
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar", is_active=False)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        user = mommy.make('User', is_superuser=True)
        arguments = {'select_box_multiple_series': m2m.id, 'select_box_dynamic_%s' % m2m_active.id: 'True'}
        serie = self.stats.get_multi_time_series(arguments, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('Bar', 0), ('Foo', 1))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series_criteria_combine_user_exception(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        If user has no permission and user field is not defined, exception must be thrown.
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="last_name",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        user = mommy.make('User')
        arguments = {'select_box_multiple_series': m2m.id}
        with self.assertRaisesRegex(Exception, "^User field must be defined to enable charts for non-superusers$"):
            self.stats.get_multi_time_series(arguments, time_since, time_until, interval, None, None, user)

    @skipIf(django.VERSION[0] < 3, "Django < 3 doesn't support Sum")
    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series_criteria_user(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        Check results, if stats are displayed for user
        """
        stats = mommy.make(
            'DashboardStats',
            model_name="TestKid",
            date_field_name='appointment',
            model_app_name="demoproject",
            type_operation_field_name="Sum",
            distinct=True,
            operation_field_name='age',
            user_field_name='author',
        )
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="name",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats, use_as='multiple_series')
        user = mommy.make('User')
        mommy.make('TestKid', appointment=datetime.date(2010, 10, 12), name="Foo", age=5, author=user)
        mommy.make('TestKid', appointment=datetime.date(2010, 10, 13), name="Bar", age=7, author=user)
        mommy.make('TestKid', appointment=datetime.date(2010, 10, 13), name="Bar", age=7)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        arguments = {'select_box_multiple_series': m2m.id}
        serie = stats.get_multi_time_series(arguments, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0, tzinfo=datetime.timezone.utc): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 11, 0, 0, tzinfo=datetime.timezone.utc): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 12, 0, 0, tzinfo=datetime.timezone.utc): OrderedDict((('Bar', None), ('Foo', 5))),
            datetime.datetime(2010, 10, 13, 0, 0, tzinfo=datetime.timezone.utc): OrderedDict((('Bar', 7), ('Foo', None))),
            datetime.datetime(2010, 10, 14, 0, 0, tzinfo=datetime.timezone.utc): OrderedDict((('Bar', 0), ('Foo', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(django.VERSION[0] < 3, "Django < 3 doesn't support Avg")
    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series_criteria_isnull(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        Check __isnull criteria
        """
        stats = mommy.make(
            'DashboardStats',
            model_name="TestKid",
            date_field_name='appointment',
            model_app_name="demoproject",
            type_operation_field_name="Avg",
            distinct=True,
            operation_field_name='age',
        )
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="birthday",
            dynamic_criteria_field_name="birthday__isnull",
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats, use_as='multiple_series')
        mommy.make('TestKid', appointment=datetime.date(2010, 10, 12), birthday=datetime.date(2010, 11, 12), age=4)
        mommy.make('TestKid', appointment=datetime.date(2010, 10, 13), birthday=None, age=3)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        arguments = {'select_box_multiple_series': m2m.id}
        user = mommy.make('User', is_staff=True)
        serie = stats.get_multi_time_series(arguments, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0, tzinfo=datetime.timezone.utc): {'Blank': 0, 'Non blank': 0},
            datetime.datetime(2010, 10, 11, 0, 0, tzinfo=datetime.timezone.utc): {'Blank': 0, 'Non blank': 0},
            datetime.datetime(2010, 10, 12, 0, 0, tzinfo=datetime.timezone.utc): {'Blank': None, 'Non blank': 4},
            datetime.datetime(2010, 10, 13, 0, 0, tzinfo=datetime.timezone.utc): {'Blank': 3, 'Non blank': None},
            datetime.datetime(2010, 10, 14, 0, 0, tzinfo=datetime.timezone.utc): {'Blank': 0, 'Non blank': 0},
        }
        self.assertDictEqual(serie, testing_data)

    @skipIf(settings.DATABASES['default']['ENGINE'] == 'django.db.backends.mysql', 'no support of USE_TZ=False in mysql')
    @override_settings(USE_TZ=False)
    def test_get_multi_series_fixed_criteria(self):
        """
        Test function to check DashboardStats.get_multi_time_series()
        Try to combine multiple_series filter with chart_filter.
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="last_name",
        )
        criteria_active = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            criteria_fix_mapping={"is_active": True},
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        mommy.make('CriteriaToStatsM2M', criteria=criteria_active, stats=self.stats, use_as='chart_filter')
        mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo", is_active=True)
        mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar", is_active=False)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)

        interval = "days"
        arguments = {'select_box_multiple_series': m2m.id}
        user = mommy.make('User', is_superuser=True)
        serie = self.stats.get_multi_time_series(arguments, time_since, time_until, interval, None, None, user)
        testing_data = {
            datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 12, 0, 0): OrderedDict((('Bar', 0), ('Foo', 1))),
            datetime.datetime(2010, 10, 13, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
            datetime.datetime(2010, 10, 14, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
        }
        self.assertDictEqual(serie, testing_data)

    def test_no_user_field_name(self):
        """
        Test that non-superuser without user_field_name can't see charts
        """
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="name",
            dynamic_criteria_field_name="last_name",
        )
        mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            criteria_fix_mapping={"is_active": True},
        )
        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
        user = mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo", is_active=True)
        time_since = datetime.datetime(2010, 10, 10)
        time_until = datetime.datetime(2010, 10, 14)
        arguments = {'select_box_multiple_series': m2m.id}
        with self.assertRaises(Exception):
            self.stats.get_multi_time_series(arguments, time_since, time_until, "days", None, user)


class SuperuserViewsTests(BaseSuperuserAuthenticatedClient):
    def setUp(self):
        self.stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
        )
        super().setUp()

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series(self):
        """Test function view rendering multi series"""
        mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=datetime.timezone.utc))
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += "?time_since=2010-10-08&time_until=2010-10-12&select_box_interval=days&select_box_chart_type=discreteBarChart"
        response = self.client.get(url)
        assertContainsAny(self, response, ('{"x": 1286668800000, "y": 1}', '{"y": 1, "x": 1286668800000}'))

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series_dynamic_criteria(self):
        """Test function view rendering multi series with dynamic criteria"""
        criteria = mommy.make(
            'DashboardStatsCriteria',
            criteria_name="active",
            dynamic_criteria_field_name="is_active",
            criteria_dynamic_mapping={
                "": [None, "All"],
                "false": [True, "Inactive"],
                "true": [False, "Active"],
            },
        )
        mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series', id=5)
        mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=datetime.timezone.utc))
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += "?time_since=2010-10-08"
        url += "&time_until=2010-10-12"
        url += "&select_box_interval=days"
        url += "&select_box_chart_type=discreteBarChart"
        url += "&select_box_multiple_series=5"
        url += "&debug=True"
        response = self.client.get(url)
        self.assertContains(response, ('"key": "Inactive"'))
        self.assertContains(response, ('"key": "Active"'))


class UserViewsTests(BaseUserAuthenticatedClient):
    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_no_permissions_not_enabled(self):
        mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            user_field_name=None,
            show_to_users=False,
        )
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += "?time_since=2010-10-08&time_until=2010-10-12&select_box_interval=days&select_box_chart_type=discreteBarChart"
        response = self.client.get(url)
        self.assertContains(response, "You have no permission to view this chart. Check if you are logged in")

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_no_permissions(self):
        mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            user_field_name=None,
            show_to_users=True,
        )
        permission = Permission.objects.get(codename='view_dashboardstats')
        self.user.user_permissions.add(permission)
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += "?time_since=2010-10-08&time_until=2010-10-12&select_box_interval=days&select_box_chart_type=discreteBarChart"
        response = self.client.get(url)
        assertContainsAny(self, response, ('{"x": 1286668800000, "y": 0}', '{"y": 0, "x": 1286668800000}'))

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_user_chart(self):
        mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            user_field_name=None,
            show_to_users=True,
        )
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += "?time_since=2010-10-08&time_until=2010-10-12&select_box_interval=days&select_box_chart_type=discreteBarChart"
        response = self.client.get(url)
        assertContainsAny(self, response, ('{"x": 1286668800000, "y": 0}', '{"y": 0, "x": 1286668800000}'))


class AdminToolsStatsModel(TestCase):
    """
    Test DashboardStatsCriteria, DashboardStats models
    """
    def setUp(self):
        # DashboardStatsCriteria model
        self.dashboard_stats_criteria = DashboardStatsCriteria(
            criteria_name="call_type",
            criteria_fix_mapping='',
            dynamic_criteria_field_name='disposition',
            criteria_dynamic_mapping={
                "INVALIDARGS": "INVALIDARGS",
                "BUSY": "BUSY",
                "TORTURE": "TORTURE",
                "ANSWER": "ANSWER",
                "DONTCALL": "DONTCALL",
                "FORBIDDEN": "FORBIDDEN",
                "NOROUTE": "NOROUTE",
                "CHANUNAVAIL": "CHANUNAVAIL",
                "NOANSWER": "NOANSWER",
                "CONGESTION": "CONGESTION",
                "CANCEL": "CANCEL",
            },
        )
        self.dashboard_stats_criteria.save()
        self.assertEqual(self.dashboard_stats_criteria.__str__(), 'call_type')

        # DashboardStats model
        self.dashboard_stats = mommy.make(
            'DashboardStats',
            graph_key='user_graph_test',
            graph_title='User graph',
            model_app_name='auth',
            model_name='User',
            date_field_name='date_joined',
            is_visible=1,
        )
        mommy.make('CriteriaToStatsM2M', criteria=self.dashboard_stats_criteria, stats=self.dashboard_stats, use_as='multiple_series')
        with self.assertRaises(ValidationError) as e:
            self.dashboard_stats.clean()
        self.assertEqual(e.exception.message_dict, {})
        self.assertEqual(self.dashboard_stats.__str__(), 'user_graph_test')

    def test_dashboard_criteria(self):
        self.assertEqual(self.dashboard_stats_criteria.criteria_name, "call_type")
        self.assertEqual(self.dashboard_stats.graph_key, 'user_graph_test')

    def teardown(self):
        self.dashboard_stats_criteria.delete()
        self.dashboard_stats.delete()



from django.test import TestCase
from model_mommy import mommy

from admin_tools_stats.forms import ChartSettingsForm


class ChartSettingsFormTests(TestCase):

    def test_operations_list(self):
        stats = mommy.make(
            'DashboardStats',
            allowed_type_operation_field_name=['Sum', 'Count'],
            operation_field_name="auth,user",
        )
        ch = ChartSettingsForm(stats)
        self.assertEqual(ch.fields['select_box_operation_field'].choices, [('', '(divide all)'), ('auth', 'auth'), ('user', 'user')])



from django.test.utils import override_settings
from django.urls import reverse
from model_mommy import mommy

from .utils import BaseSuperuserAuthenticatedClient


class AdminIndexTests(BaseSuperuserAuthenticatedClient):
    def setUp(self):
        self.stats = mommy.make(
            'DashboardStats',
            graph_title="User chart",
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            operation_field_name='is_active,is_staff',
        )
        super().setUp()

    @override_settings(
        INSTALLED_APPS=[
            'django_nvd3',
            'admin_tools_stats',
            'admin_tools.menu',
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.messages',
            'django.contrib.staticfiles',
            'djangobower',
            'demoproject',
        ]
    )
    def test_admin_index(self):
        """Test vanila admin index page, that should contain chart"""
        url = reverse('admin:index')
        response = self.client.get(url)
        self.assertContains(response, "<h3>User chart</h3>", html=True)
        self.assertContains(
            response,
            '<select name="select_box_operation_field" class="chart-input" required>'
            '<option value="">(divide all)</option>'
            '<option value="is_active" selected>is_active</option>'
            '<option value="is_staff">is_staff</option>'
            '</select>',
            html=True,
        )



from django.contrib.auth.models import User
from django.test import Client, TestCase
from django.test.client import RequestFactory


class BaseSuperuserAuthenticatedClient(TestCase):
    """Common Authentication"""
    fixtures = ['auth_user']
    username = 'admin'

    def setUp(self):
        """To create admin user"""
        self.client = Client()

        self.user = User.objects.get(username=self.username)
        self.client.force_login(self.user)

        self.factory = RequestFactory()


class BaseUserAuthenticatedClient(BaseSuperuserAuthenticatedClient):
    username = 'user_1'


def assertContainsAny(
        self, response, texts, status_code=200,
        msg_prefix='', html=False,
):
    total_count = 0
    for text in texts:
        text_repr, real_count, msg_prefix = self._assert_contains(response, text, status_code, msg_prefix, html)
        total_count += real_count

    self.assertTrue(total_count != 0, f"None of the {texts} were found in the response: {response}")



# Generated by Django 3.0.11 on 2021-01-22 16:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admin_tools_stats', '0009_auto_20200928_1003'),
    ]

    operations = [
        migrations.AddField(
            model_name='dashboardstats',
            name='show_to_users',
            field=models.BooleanField(default=False, help_text="Be carefull and test if it still doesn't expose sensitive data", verbose_name='show to ordinary users'),
        ),
    ]



from datetime import timedelta

from django import forms
from django.utils.timezone import now


class ChartSettingsForm(forms.Form):
    def __init__(self, stats, user=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for ch_filter in stats.criteriatostatsm2m_set.filter(use_as='chart_filter').order_by('order'):
            dy_map = ch_filter.get_dynamic_choices(user=user)
            if dy_map:
                self.fields[f'select_box_dynamic_{ch_filter.id}'] = forms.ChoiceField(
                    choices=[('', '-------')] + list(dy_map.values()),
                    label=ch_filter.criteria.criteria_name,
                    initial=ch_filter.default_option,
                )
                self.fields[f'select_box_dynamic_{ch_filter.id}'].widget.attrs['class'] = "chart-input"

        self.fields['graph_key'] = forms.CharField(initial=stats.graph_key, widget=forms.HiddenInput(attrs={'class': 'hidden_graph_key'}))

        multiple_series = stats.criteriatostatsm2m_set.filter(use_as='multiple_series')
        if multiple_series.exists():
            choices = multiple_series.select_related('stats__default_multiseries_criteria', 'criteria'). \
                order_by('order').values_list('id', 'criteria__criteria_name')
            self.fields['select_box_multiple_series'] = forms.ChoiceField(
                label="Divide",
                choices=[('', '-------')] + list(choices),
                initial=stats.default_multiseries_criteria.id if stats.default_multiseries_criteria else None,
            )
            self.fields['select_box_multiple_series'].widget.attrs['class'] = "chart-input select_box_multiple_series"

        if len(stats.allowed_type_operation_field_name) > 1:
            self.fields['select_box_operation'] = forms.ChoiceField(
                choices=stats.allowed_type_operation_field_name_choices(),
                label="Operation",
                initial=stats.type_operation_field_name,
            )
            self.fields['select_box_operation'].widget.attrs['class'] = "chart-input"

        operations_list = stats.get_operations_list()
        if operations_list and len(operations_list) > 1:
            self.fields['select_box_operation_field'] = forms.ChoiceField(
                choices=[('', '(divide all)')] + [(o, o) for o in operations_list],
                label="Field",
                initial=operations_list[0],
            )
            self.fields['select_box_operation_field'].widget.attrs['class'] = "chart-input"

        if len(stats.allowed_time_scales) > 1:
            self.fields['select_box_interval'] = forms.ChoiceField(
                choices=stats.allowed_time_scales_choices(),
                label="Scale",
                initial=stats.default_time_scale,
            )
            self.fields['select_box_interval'].widget.attrs['class'] = "chart-input"

        if len(stats.allowed_chart_types) > 1:
            self.fields['select_box_chart_type'] = forms.ChoiceField(
                choices=stats.allowed_chart_types_choices(),
                label="Chart",
                initial=stats.default_chart_type,
            )
            self.fields['select_box_chart_type'].widget.attrs['class'] = "chart-input select_box_chart_type"

        self.fields['time_since'] = forms.DateField(
            label='Since',
            initial=(now() - timedelta(days=stats.default_time_period)).strftime('%Y-%m-%d'),
            widget=forms.TextInput(attrs={'type': 'date', 'class': 'chart-input select_box_date_since'}),
        )
        self.fields['time_until'] = forms.DateField(
            label='Until',
            initial=now().strftime('%Y-%m-%d'),
            widget=forms.TextInput(attrs={'type': 'date', 'class': 'chart-input select_box_date_until'}),
        )




from django.urls import path

from . import views

urlpatterns = [
    path('admin_charts.js', views.AdminChartsView.as_view(content_type='application/javascript'), name='admin-charts'),
    path('chart_data/', views.ChartDataView.as_view(), name='chart-data'),  # Only to get the base address in template, will need parameters
    path('chart_data/<str:graph_key>/', views.ChartDataView.as_view(), name='chart-data'),
    path('analytics/', views.AnalyticsView.as_view(), name='chart-analytics'),
    path('analytics/chart/<str:graph_key>/', views.AnalyticsChartView.as_view(), name='chart-analytics'),
]



# Generated by Django 3.0.11 on 2021-02-04 11:06

import multiselectfield.db.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin_tools_stats', '0010_dashboardstats_show_to_users'),
    ]

    operations = [
        migrations.AddField(
            model_name='dashboardstats',
            name='allowed_chart_types',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('discreteBarChart', 'Bar'), ('lineChart', 'Line'), ('multiBarChart', 'Multi Bar'), ('pieChart', 'Pie'), ('stackedAreaChart', 'Stacked Area'), ('multiBarHorizontalChart', 'Multi Bar Horizontal'), ('linePlusBarChart', 'Line Plus Bar'), ('scatterChart', 'Scatter'), ('cumulativeLineChart', 'Cumulative Line'), ('lineWithFocusChart', 'Line With Focus')], default=('discreteBarChart', 'lineChart', 'multiBarChart', 'pieChart', 'stackedAreaChart', 'multiBarHorizontalChart', 'linePlusBarChart', 'scatterChart', 'cumulativeLineChart', 'lineWithFocusChart'), max_length=1000, null=True, verbose_name='Allowed chart types'),
        ),
        migrations.AddField(
            model_name='dashboardstats',
            name='allowed_time_scales',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default=('hours', 'days', 'weeks', 'months', 'years'), max_length=1000, verbose_name='Chart type'),
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-21 11:47

import multiselectfield.db.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admin_tools_stats', '0012_auto_20210207_0859'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dashboardstats',
            name='allowed_time_scales',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('quarters', 'Quarters'), ('years', 'Years')], default=('hours', 'days', 'weeks', 'months', 'quarters', 'years'), max_length=1000, verbose_name='Allowed time scales'),
        ),
        migrations.AlterField(
            model_name='dashboardstats',
            name='default_time_scale',
            field=models.CharField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('quarters', 'Quarters'), ('years', 'Years')], default='days', max_length=90, verbose_name='Default time scale'),
        ),
    ]



# Generated by Django 3.2.10 on 2021-12-28 06:27

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('demoproject', '0002_auto_20210221_0541'),
    ]

    operations = [
        migrations.AddField(
            model_name='testkid',
            name='author',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (C) 2011-2014 Star2Billing S.L.
#
# The Initial Developer of the Original Code is
# Arezqui Belaid <info@star2billing.com>
#
import datetime

import django
from django.test.utils import override_settings
from django.urls import reverse
from model_mommy import mommy

from admin_tools_stats.views import AnalyticsView

from .utils import BaseSuperuserAuthenticatedClient, assertContainsAny


class AnalyticsViewTest(BaseSuperuserAuthenticatedClient):

    def setUp(self):
        self.stats = mommy.make(
            'DashboardStats',
            graph_title="User chart",
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            allowed_type_operation_field_name=['Sum', 'Count'],
        )
        self.kid_stats = mommy.make(
            'DashboardStats',
            graph_title="Kid chart",
            date_field_name="birthday",
            model_name="TestKid",
            model_app_name="demoproject",
            graph_key="kid_graph",
        )
        super().setUp()

    def test_analytics_view(self):
        """Test function to check dashboardstats admin pages"""
        response = self.client.get(reverse('chart-analytics'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "<button>Kid chart</button>", html=True)

    def test_analytics_chart_view(self):
        """Test function to check dashboardstats admin pages"""
        response = self.client.get(reverse('chart-analytics', kwargs={'graph_key': 'user_graph'}))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "<h3>User chart</h3>", html=True)
        self.assertContains(
            response,
            '<select name="select_box_operation" class="chart-input">'
            '<option value="Count">Count</option>'
            '<option value="Sum">Sum</option>'
            '</select>',
            html=True)

    def test_get_charts_query(self):
        a = AnalyticsView()
        a.request = self.client.request()
        a.request.user = mommy.make("User", is_superuser=True)
        if django.VERSION > (3, 2):
            self.assertQuerysetEqual(a.get_charts_query(), [self.kid_stats, self.stats])
        else:
            self.assertQuerysetEqual(a.get_charts_query(), ['<DashboardStats: kid_graph>', '<DashboardStats: user_graph>'])

    def test_get_charts_query_usser(self):
        a = AnalyticsView()
        kid_graph_user = mommy.make(
            'DashboardStats',
            graph_title="Kid chart",
            date_field_name="birthday",
            model_name="TestKid",
            model_app_name="demoproject",
            graph_key="kid_graph_user",
            show_to_users=True,
        )
        a.request = self.client.request()
        a.request.user = mommy.make("User")
        if django.VERSION > (3, 2):
            self.assertQuerysetEqual(a.get_charts_query(), [kid_graph_user])
        else:
            self.assertQuerysetEqual(a.get_charts_query(), ['<DashboardStats: kid_graph_user>'])

    def test_get_templates_names(self):
        a = AnalyticsView()
        a.request = self.client.request()
        a.request.user = mommy.make("User", is_superuser=True)
        self.assertEqual(a.get_template_names(), 'admin_tools_stats/analytics.html')

    def test_get_templates_names_usser(self):
        a = AnalyticsView()
        a.request = self.client.request()
        a.request.user = mommy.make("User")
        self.assertEqual(a.get_template_names(), 'admin_tools_stats/analytics_user.html')


class MultiFieldViewsTests(BaseSuperuserAuthenticatedClient):
    def setUp(self):
        self.stats = mommy.make(
            'DashboardStats',
            date_field_name="date_joined",
            model_name="User",
            model_app_name="auth",
            graph_key="user_graph",
            operation_field_name='is_active,is_staff',
        )
        super().setUp()

    @override_settings(USE_TZ=True, TIME_ZONE='UTC')
    def test_get_multi_series_multiple_operations(self):
        """Test function view rendering multi series with multiple operations"""
        mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=datetime.timezone.utc))
        url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
        url += (
            "?time_since=2010-10-08&time_until=2010-10-12&select_box_interval=days&"
            "select_box_chart_type=discreteBarChart&select_box_operation_field="
        )
        response = self.client.get(url)
        assertContainsAny(self, response, ('{"x": 1286668800000, "y": 1}', '{"y": 1, "x": 1286668800000}'))



# Generated by Django 3.1.6 on 2021-02-07 07:59

import multiselectfield.db.fields
from django.db import migrations, models

try:
    from django.db.models import JSONField
except ImportError:
    from jsonfield.fields import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('admin_tools_stats', '0011_auto_20210204_1206'),
    ]

    operations = [
        migrations.AddField(
            model_name='dashboardstats',
            name='allowed_type_operation_field_name',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('Count', 'Count'), ('Sum', 'Sum'), ('Avg', 'Avgerage'), ('AvgCountPerInstance', 'Avgerage count per active model instance'), ('Max', 'Max'), ('Min', 'Min'), ('StdDev', 'StdDev'), ('Variance', 'Variance')], help_text='choose the type operation what you want to aggregate, ex. Sum', max_length=1000, null=True, verbose_name='Choose Type operation'),
        ),
        migrations.AlterField(
            model_name='dashboardstats',
            name='allowed_time_scales',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default=('hours', 'days', 'weeks', 'months', 'years'), max_length=1000, verbose_name='Allowed time scales'),
        ),
        migrations.AlterField(
            model_name='dashboardstatscriteria',
            name='criteria_dynamic_mapping',
            field=JSONField(blank=True, help_text='a JSON dictionary with records in two following possible formats:<br/>"key_value": "name"<br/>"key": [value, "name"]<br/>use blank key for no filter<br/>Example:<br/><pre>{<br/>  "": [null, "All"],<br/>  "True": [true, "True"],<br/>  "False": [false, "False"]<br/>}</pre><br/>Left blank to exploit all choices of CharField with choices', null=True, verbose_name='dynamic criteria / value'),
        ),
        migrations.AlterField(
            model_name='dashboardstatscriteria',
            name='criteria_fix_mapping',
            field=JSONField(blank=True, help_text='a JSON dictionary of key-value pairs that will be used for the criteria', null=True, verbose_name='fixed criteria / value'),
        ),
    ]
